from collections import OrderedDict

from django.conf import settings
from django.db import models
from django.utils.functional import cached_property

from register.models.attendee import Attendee


class Meal(models.Model):
    MEALS = OrderedDict((
        ('breakfast', 'Breakfast'),
        ('lunch', 'Lunch'),
        ('dinner', 'Dinner'),
    ))

    date = models.DateField(db_index=True)
    meal = models.CharField(max_length=16)

    @cached_property
    def form_name(self):
        return '{}_{}'.format(self.meal, self.date.isoformat())

    @cached_property
    def conference_dinner(self):
        return (self.meal == 'dinner'
            and self.date == settings.DEBCONF_CONFERENCE_DINNER_DAY)

    def __str__(self):
        return '{}: {}'.format(self.date.isoformat(), self.meal)

    class Meta:
        ordering = ['date']
        unique_together = ('date', 'meal')


class Food(models.Model):
    DIETS = OrderedDict((
        ('', 'I will be happy to eat whatever is provided'),
        ('vegetarian', "I am lacto-ovo vegetarian, don't provide "
                       "meat/fish for me"),
        ('vegan', "I am strict vegetarian (vegan), don't provide any "
                  "animal products for me"),
        ('gluten_free', 'I require gluten-free food'),
        ('other', 'Other, described below'),
    ))

    attendee = models.OneToOneField(Attendee, related_name='food',
                                    on_delete=models.CASCADE)

    meals = models.ManyToManyField(Meal)
    diet = models.CharField(max_length=16, blank=True)
    special_diet = models.TextField(blank=True)

    def __str__(self):
        return 'Attendee <{}>'.format(self.attendee.user.username)

    @cached_property
    def meals_by_day(self):
        meal_order = list(Meal.MEALS.keys())
        meals = sorted(
            self.meals.all(),
            key=lambda meal: (meal.date, meal_order.index(meal.meal)))

        by_day = OrderedDict()
        for meal in meals:
            by_day.setdefault(meal.date, []).append(meal)
        return by_day



import pytest
from django.contrib.auth.models import User
from wafer.talks.models import Talk, TalkUrl
from debconf.talk_urls import create_online_service_urls


@pytest.fixture(autouse=True)
def url_settings(settings):
    settings.DEBCONF_TALK_PROVISION_URLS = {
        "etherpad": {
            "pattern": "https://pad.online.debconf.org/p/{id}-{slug:.32}",
            "public": True,
        },
        "jitsi": {
            "pattern": "https://jitsi.debian.social/{id}-{slug}-{secret16}",
            "public": False,
        },
    }


@pytest.fixture
def talk(db):
    author = User.objects.create(username="author")
    return Talk.objects.create(title="Hello World", corresponding_author=author)


def etherpad(talk):
    return TalkUrl.objects.get(talk=talk, description="etherpad")


def jitsi(talk):
    return TalkUrl.objects.get(talk=talk, description="jitsi")


class TestCreateOnlineServiceUrls:
    def test_id_and_slug(self, talk):
        create_online_service_urls(talk)
        assert (
            etherpad(talk).url
            == f"https://pad.online.debconf.org/p/{talk.pk}-hello-world"
        )

    def test_secret16(self, talk, mocker):
        mocker.patch(
            "debconf.talk_urls.get_random_string", return_value="01234567890abcdef"
        )
        create_online_service_urls(talk)
        assert (
            jitsi(talk).url
            == f"https://jitsi.debian.social/{talk.pk}-hello-world-01234567890abcdef"
        )

    def test_public(self, talk):
        create_online_service_urls(talk)
        assert etherpad(talk).public

    def test_private(self, talk):
        create_online_service_urls(talk)
        assert not jitsi(talk).public



from collections import defaultdict
import csv
import datetime

from django.core.management.base import BaseCommand

from wafer.schedule.models import Day, Slot, Venue, ScheduleItem
from wafer.talks.models import Talk, CANCELLED, ACCEPTED


def parse_timestamp(timestamp):
    """Parse a timestamp into a usable datetime.datetime object"""
    hour, minute = timestamp.split(':')
    return datetime.datetime(1970, 1, 1, int(hour), int(minute))


def unparse_timestamp(dt):
    """Unparse a parsed timestamp"""
    return dt.time()


class Command(BaseCommand):
    help = 'Import schedule from a CSV grid'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')
        parser.add_argument('--accept-talks', action='store_true',
                            help='Accept the talks that need accepting')
        parser.add_argument('file', help='CSV file')

    def parse_csv(self, csv):
        return

    def handle(self, *args, **options):
        do_it = options['yes']
        accept_talks = options['accept_talks']
        filename = options['file']
        with open(filename, 'r') as f:
            dialect = csv.Sniffer().sniff(f.read(4096))
            f.seek(0)
            data = list(csv.DictReader(f, dialect=dialect))

        rooms = [key for key in data[0] if key.startswith('room_')]
        room_map = {
            room: Venue.objects.get(pk=int(room[5:]))
            for room in rooms
        }
        slots = []
        talk_venue_slots = defaultdict(list)

        lineno = 0
        while lineno < len(data) - 1:
            half_hr_1 = data[lineno]
            half_hr_2 = data[lineno + 1]
            day = Day.objects.get(date__day=int(half_hr_1['day']))
            start_time_1 = parse_timestamp(half_hr_1['start_time'])
            start_time_2 = parse_timestamp(half_hr_2['start_time'])

            # Check whether the slot should be merged with the next one
            half_hour_slots = False
            for room in rooms:
                if not half_hr_1[room] and not half_hr_2[room]:
                    continue
                if half_hr_1[room] != half_hr_2[room]:
                    half_hour_slots = True

            if half_hour_slots:
                slot1, _ = Slot.objects.get_or_create(
                    day=day,
                    start_time=unparse_timestamp(start_time_1),
                    end_time=unparse_timestamp(
                        start_time_1 + datetime.timedelta(seconds=20*60)
                    ),
                    name='First Half Hour',
                )
                slot_middle, _ = Slot.objects.get_or_create(
                    day=day,
                    start_time=slot1.end_time,
                    end_time=unparse_timestamp(start_time_2),
                    name='Short Break',
                )
                slot2, _ = Slot.objects.get_or_create(
                    day=day,
                    start_time=slot_middle.end_time,
                    end_time=unparse_timestamp(
                        start_time_2 + datetime.timedelta(seconds=20*60)
                    ),
                    name='Second Half Hour',
                )
                slot_filler, _ = Slot.objects.get_or_create(
                    day=day,
                    start_time=slot2.end_time,
                    end_time=unparse_timestamp(
                        start_time_1 + datetime.timedelta(seconds=60*60)
                    ),
                    name='Less Short Break',
                )
                all_slots = [slot1, slot_middle, slot2, slot_filler]
                slots.extend(all_slots)

                for room in rooms:
                    if half_hr_1[room]:
                        talk_venue_slots[half_hr_1[room], room].extend(
                            (slot1, slot_middle)
                        )
                    if half_hr_2[room]:
                        talk_venue_slots[half_hr_2[room], room].extend(
                            (slot2, slot_filler)
                        )
            else:
                slot2, _ = Slot.objects.get_or_create(
                    day=day,
                    start_time=unparse_timestamp(start_time_1),
                    end_time=unparse_timestamp(
                        start_time_1 + datetime.timedelta(seconds=45*60)
                    ),
                    name='Full Hour',
                )
                slot_filler, _ = Slot.objects.get_or_create(
                    day=day,
                    start_time=slot2.end_time,
                    end_time=unparse_timestamp(
                        start_time_1 + datetime.timedelta(seconds=60*60)
                    ),
                    name='Less Short Break',
                )
                slots.append(slot2)
                slots.append(slot_filler)
                for room in rooms:
                    if not half_hr_1[room]:
                        continue
                    talk_venue_slots[half_hr_1[room], room].extend(
                        (slot2, slot_filler)
                    )

            print(day, half_hr_1, half_hr_2)

            lineno += 2

        for slot in slots:
            slot.clean()
            print(slot.day, slot.start_time, slot.end_time)
            if do_it:
                slot.save()

        for (talk, venue), slots in talk_venue_slots.items():
            talk = Talk.objects.get(pk=int(talk))
            print(talk, venue, slots)
            if do_it:
                s_i, _ = ScheduleItem.objects.get_or_create(
                    talk=talk, defaults={'venue': room_map[venue]}
                )
                s_i.venue = room_map[venue]
                s_i.clean()
                if accept_talks and talk.status not in (CANCELLED, ACCEPTED):
                    talk.status = ACCEPTED
                    talk.save()
                s_i.save()
                s_i.slots.set(slots[:-1])
                print(s_i, s_i.slots.all())



from django import forms
from django.conf import settings

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Fieldset, HTML, Layout

from bursary.models import Bursary


BURSARIES_LINK = (
    '<a href="/about/bursaries/" target="blank">DebConf bursary instructions.'
    '</a>')


class BursaryForm(forms.Form):
    request_travel = forms.BooleanField(
        label='I want to apply for a travel bursary',
        required=False,
    )
    request_food = forms.BooleanField(
        label='I want to apply for a food bursary',
        required=False,
    )
    request_accommodation = forms.BooleanField(
        label='I want to apply for an accommodation bursary',
        required=False,
    )
    request_expenses = forms.BooleanField(
        label='I want to apply for an expenses reimbursement bursary',
        required=False,
    )
    reason_contribution = forms.CharField(
        label='My contributions to Debian',
        widget=forms.Textarea(attrs={'rows': 5}),
        required=False,
        help_text='To help us evaluate your eligibility for a Debian bursary.',
    )
    reason_plans = forms.CharField(
        label='My plans for DebCamp or DebConf',
        help_text='To help us evaluate your eligibility for a Debian bursary.',
        widget=forms.Textarea(attrs={'rows': 5}),
        required=False,
    )
    reason_diversity = forms.CharField(
        label='My eligibility for a diversity bursary',
        widget=forms.Textarea(attrs={'rows': 5}),
        help_text='Diversity bursary applications only. Please consult the '
                  '<a href="/about/bursaries/#diversity-bursaries" '
                  'target="blank">diversity bursary instructions</a>.',
        required=False,
    )
    need = forms.ChoiceField(
        label='My level of need',
        choices=(
            ('', 'N/A (not requesting a travel bursary)'),
            ('unable', Bursary.BURSARY_NEEDS['unable']),
            ('sacrifice', Bursary.BURSARY_NEEDS['sacrifice']),
            ('inconvenient', Bursary.BURSARY_NEEDS['inconvenient']),
            ('non-financial', Bursary.BURSARY_NEEDS['non-financial']),
        ),
        required=False,
    )
    travel_bursary = forms.IntegerField(
        label=f'My travel expense claim (in '
              f'{settings.DEBCONF_BURSARY_CURRENCY})',
        help_text='Estimated amount required. ' + BURSARIES_LINK,
        min_value=0,
        max_value=10000,
        required=False,
    )
    travel_from = forms.CharField(
        label="I'm travelling from",
        help_text='Knowing where you need to travel from helps us evaluate '
                  'the amount you are claiming.',
        required=False,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False

        readonly_attrs = {}
        if settings.BURSARIES_CLOSED:
            readonly_attrs['readonly'] = 'readonly'

        if settings.DEBCONF_ONLINE:
            requests = (
                Field('request_expenses', id='bursary-request-expenses'),
            )
            travel = ()
        else:
            requests = (
                Field('request_food', id='bursary-request-food'),
                Field('request_accommodation', id='bursary-request-accomm'),
                Field('request_travel', id='bursary-request-travel'),
            )
            travel = (
                Fieldset(
                    'Travel Bursary Details',
                    'travel_bursary',
                    Field('travel_from', **readonly_attrs),
                    Field('need', readonly=settings.BURSARIES_CLOSED),
                    css_id='travel-details',
                ),
            )

        self.helper.layout = Layout(
            Fieldset(
                'Bursary Requests',
                *requests,
                css_id='bursary-requests',
            ),
            Fieldset(
                'Bursary Details',
                HTML('<p>This is where you explain your needs, and '
                     'involvement in Debian, that justify a bursary. See the '
                     + BURSARIES_LINK + '.</p>'),
                Field(
                    'reason_contribution', **readonly_attrs),
                Field('reason_plans', **readonly_attrs),
                Field('reason_diversity', **readonly_attrs),
                css_id='bursary-details',
            ),
            *travel
        )
        if settings.BURSARIES_CLOSED:
            self.helper.layout.insert(0, HTML(
                '<div class="alert alert-warning">'
                '<strong>Bursaries are closed</strong> '
                'You can reduce or cancel your bursary request. '
                'But no other changes are possible any more.'
                '</div>'
            ))

    def clean_travel_bursary(self):
        travel_bursary = self.cleaned_data.get('travel_bursary')
        if travel_bursary == 0:
            return None
        return travel_bursary

    def clean(self):
        cleaned_data = super().clean()

        request_food = cleaned_data.get('request_food')
        request_accommodation = cleaned_data.get('request_accommodation')
        request_travel = cleaned_data.get('request_travel')
        request_expenses = cleaned_data.get('request_expenses')
        if request_food or request_accommodation or request_travel or request_expenses:
            if not cleaned_data.get('reason_plans'):
                self.add_error(
                    'reason_plans',
                    'Please share your plans for the conference, when applying '
                    'for a bursary.')

            if (not cleaned_data.get('reason_contribution')
                    and not cleaned_data.get('reason_diversity')):
                for field in ('reason_contribution', 'reason_diversity'):
                    self.add_error(
                        field,
                        'Please describe your contributions and/or the '
                        'diversity of your background, when applying for a '
                        'bursary.')

        if request_travel:
            for field in ('travel_bursary', 'travel_from'):
                if not cleaned_data.get(field):
                    self.add_error(
                        field,
                        'Please share your travel details, when applying for '
                        'a travel bursary.'
                    )
            if not cleaned_data.get('need'):
                self.add_error(
                    'need',
                    'Please share your level of need, when applying for a '
                    'travel bursary.'
                )
        else:
            cleaned_data['travel_bursary'] = None



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.contrib.sites.models import Site
from django.template import Context, Template
from django.utils.timezone import now

from debconf.tz import aoe_datetime
from register.models import Attendee
from wafer.users.models import UserProfile


SUBJECT = '{{ WAFER_CONFERENCE_NAME }}: Registration Reminder'
TXT = '''Hi,

{% if incomplete_registration %}We see that you started a registration on https://{{ WAFER_CONFERENCE_DOMAIN }}
but have not yet completed it.
{% else %}We see that you have created an account on https://{{ WAFER_CONFERENCE_DOMAIN }}
but have not yet actually registered to attend the conference.
{% endif %}
If you are intending to attend, you need to go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/>
and fill out every page of the form.

When you are successfully registered, you'll receive a confirmation e-mail, and
see a big green "Registered" label on your profile.

{% if incomplete_registration %}If you no longer plan to attend {{ WAFER_CONFERENCE_NAME }}, go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/unregister/>
and click the "Unregister" button.

{% endif %}{% if before_bursary_deadline %}Bursary applications close on {{ DEBCONF_BURSARY_DEADLINE|date:"SHORT_DATE_FORMAT" }} AoE.
If you are applying for a bursary, you need to complete your registration
before this date.
AoE means Anywhere on Earth: You don't have to think about timezones, just
apply before this date, wherever you are in the world.

We recommend that you register as soon as possible.
{% else %}Unfortunately, bursary applications have already closed.

We recommend that you register as soon as possible, if you're planning to
attend at your own cost.
{% endif %}If you need to amend your registration later, you can.

Hope to see you in {{ DEBCONF_CITY }},

The {{ WAFER_CONFERENCE_NAME }} team
'''


class Command(BaseCommand):
    help = "Badger users who haven't registered for attendance"

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')
        parser.add_argument('--site', type=int, default=1,
                            help='Django site ID, default: 1')

    def badger(self, user, dry_run, site):
        bursary_deadline = settings.DEBCONF_BURSARY_DEADLINE
        before_deadline = aoe_datetime(bursary_deadline) > now()
        try:
            attendee = user.attendee
            incomplete_registration = True
        except Attendee.DoesNotExist:
            incomplete_registration = False

        context = Context({
            'before_bursary_deadline': before_deadline,
            'incomplete_registration': incomplete_registration,
            'DEBCONF_BURSARY_DEADLINE': bursary_deadline,
            'DEBCONF_CITY': settings.DEBCONF_CITY,
            'WAFER_CONFERENCE_DOMAIN': site.domain,
            'WAFER_CONFERENCE_NAME': site.name,
        })

        txt = Template(TXT).render(context)
        subject = Template(SUBJECT).render(context)
        to = user.email
        if dry_run:
            print('I would badger:', to)
            return
        email_message = EmailMultiAlternatives(subject, txt, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        site = Site.objects.get(id=1)
        for user in UserProfile.objects.all():
            if not user.is_registered():
                self.badger(user.user, dry_run, site)



# Generated by Django 2.2.27 on 2022-03-31 17:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0013_visa'),
    ]

    operations = [
        migrations.AddField(
            model_name='attendee',
            name='confirm_covid_tests',
            field=models.BooleanField(null=True),
        ),
    ]



from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import pgettext_lazy
from django_countries.fields import CountryField

from wafer.schedule.models import ScheduleBlock
from debconf.countries import Countries


INVOLVEMENT_LEVELS = (
    (0, _('Beginner')),
    (1, _('User')),
    (2, _('Contributor')),
    (4, _('Debian Maintainer (DM)')),
    (5, _('Debian Developer (DD)')),
)

GENDERS = (
    ('m', _('Male')),
    ('f', _('Female')),
    ('n', _('Non-Binary')),
)


class Registration(models.Model):
    class Meta:
        verbose_name = pgettext_lazy('conference', 'registration')
        verbose_name_plural = pgettext_lazy('conference', 'registrations')

    user = models.OneToOneField(get_user_model(), on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now_add=True)

    # A minimal of personal information, for statistics
    involvement = models.IntegerField(
        null=True,
        blank=True,
        choices=INVOLVEMENT_LEVELS,
        verbose_name=_("Level of involvement with Debian"),
    )
    gender = models.CharField(
        max_length=1,
        blank=True,
        null=True,
        choices=GENDERS,
        verbose_name=_("Gender"),
    )
    country = CountryField(
        countries=Countries,
        null=True,
        blank=True,
        verbose_name=_("Country"),
    )
    city_state = models.CharField(
        max_length=128,
        blank=True,
        verbose_name=_("City/State or Province")
    )

    # attendance info
    days =  models.ManyToManyField(
        ScheduleBlock,
        verbose_name=_('Which days you will attend'),
    )

    @property
    def full_name(self):
        if self.user:
            return self.user.get_full_name()
        else:
            return None


def is_registered(user):
    return Registration.objects.filter(user=user).exists()



from django.conf.urls import url
from django.views.generic import RedirectView

from debconf.views import (
    ContentStatisticsView, DebConfScheduleView, IndexView,
    RobotsView, StatisticsView, now_or_next,
)

urlpatterns = [
    url(r'^favicon.ico$', RedirectView.as_view(url="/static/img/favicon.ico", permanent=True)),
    url(r'^schedule/$', DebConfScheduleView.as_view(),
        name='wafer_full_schedule'),
    url(r'^now_or_next/(?P<venue_id>\d+)/$', now_or_next, name="now_or_next"),
    url(r'^robots.txt$', RobotsView.as_view()),
    url(r'^$', IndexView.as_view()),
    url(r'^statistics/$', StatisticsView.as_view()),
    url(r'^talks/statistics/$', ContentStatisticsView.as_view(),
        name='content-statistics'),
]



from django.contrib import admin
from minidebconf.models import Registration


class RegistrationAdmin(admin.ModelAdmin):
    list_display = ('user', 'involvement', 'gender', 'country')
    list_filter = ('involvement', 'gender', 'days')

admin.site.register(Registration, RegistrationAdmin)



from django.core.management.base import BaseCommand
from wafer.talks.models import Talk

import datetime
import os
import requests


class Command(BaseCommand):
    help = 'Load video URLs from sreview.debian.net'

    def add_arguments(self, parser):
        parser.add_argument(
            '--sreview-url', metavar='URL', type=str,
            default='https://sreview.debian.net/released.json',
            help='URL for the released.json file in the SReview instance',
        )

        current_year = datetime.datetime.today().year
        parser.add_argument(
            '--base-url', metavar='URL', type=str,
            default='https://meetings-archive.debian.net/pub/debian-meetings/%s' % current_year,
            help='Base URL for the videos released by sreview',
        )

        parser.add_argument(
            '--dry-run', action='store_true',
            help='Only show what would be done',
        )

    def handle(self, *args, **options):
        jsonurl = options['sreview_url']

        jsondata = requests.get(jsonurl)
        jsondata.raise_for_status()

        data = jsondata.json()

        baseurl = options['base_url'].rstrip("/")

        for entry in data['videos']:
            talk_id = int(entry['eventid'])
            talk = Talk.objects.get(pk=talk_id)
            url = baseurl + "/" + entry['video'].lstrip("/")
            if options['dry_run']:
                print('Would load video for <%s>: %s' % (talk, url))
            else:
                talk.urls.update_or_create(
                    description='Video',
                    defaults={
                        "url": url,
                    }
                )
                print('Loaded video for <%s>: %s' % (talk, url))



from register.models.accommodation import Accomm, AccommNight
from register.models.address import Address
from register.models.attendee import Attendee
from register.models.childcare import ChildCare
from register.models.food import Food, Meal
from register.models.queue import Queue, QueueSlot
from register.models.visa import Visa
import register.speaker_stay

def user_is_registered(user):
    from register.views import STEPS
    last_step = len(STEPS) - 1
    return Attendee.objects.filter(
        user=user, completed_register_steps=last_step).exists()



# Generated by Django 2.2.27 on 2022-03-16 00:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0011_add_address'),
    ]

    operations = [
        migrations.AddField(
            model_name='attendee',
            name='vaccinated',
            field=models.BooleanField(null=True),
        ),
        migrations.AddField(
            model_name='attendee',
            name='vaccination_notes',
            field=models.TextField(blank=True),
        ),
    ]



from datetime import timedelta
from django.utils import timezone
import pytest

from wafer.schedule.models import ScheduleBlock
from debconf.context_processors import expose_settings
from debconf.context_processors import is_it_debconf


class TestExposeSettings:
    def test_expose_settings(self, mocker):
        request = mocker.MagicMock()
        context = expose_settings(request)
        assert type(context) is dict


def create_schedule(confday):
     ScheduleBlock.objects.create(
         start_time=confday + timedelta(hours=10),
         end_time=confday + timedelta(hours=18)
     )


@pytest.fixture()
def now(mocker):
    n = timezone.now()
    mocker.patch("django.utils.timezone.now", return_value=n)
    return n


@pytest.fixture()
def today(now):
    return timezone.make_aware(timezone.datetime(now.year, now.month, now.day))


class TestIsItDebConf:

    def test_no_schedule_yet(self, db):
        assert is_it_debconf(None) == {}

    def test_a_month_before_the_conference(self, db, today):
        create_schedule(today + timedelta(days=30))
        assert is_it_debconf(None) == {"debconf_day": False, "debconf_soon": False}

    def test_the_day_before_the_conference(self, db, today):
        create_schedule(today + timedelta(days=1))
        assert is_it_debconf(None) == {"debconf_day": False, "debconf_soon": True}

    def test_the_day_of_the_conference(self, db, today):
        create_schedule(today)
        assert is_it_debconf(None) == {"debconf_day": True, "debconf_soon": False}



from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Fieldset, HTML, Layout
from django_countries.fields import LazyTypedChoiceField

from register.fields import RequiredCountries


class VisaForm(forms.Form):
    visa = forms.BooleanField(
        label='I require a visa to travel travel to DebConf',
        required=False,
        help_text='See <a href="/about/visas/">the visa page</a> for details',
    )
    country = LazyTypedChoiceField(
        label='My citizenship',
        help_text='I will apply for a visa as a citizen of the specified '
                  'country.',
        choices=RequiredCountries(),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False

        self.helper.layout = Layout(
            Field('visa', id='visa'),
            Fieldset(
                'Visa Details',
                HTML(
                    'Please contact the visa team at '
                    '<a href="mailto:visa@debconf.org">visa@debconf.org</a> '
                    'for advice or assistance in obtaining a visa to travel '
                    'to DebConf.'),
                'country',
                css_id='details',
            ),
        )

    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data.get('visa') and not cleaned_data.get('country'):
            self.add_error(
                'country',
                'Please provide us with your country of citizenship.')

        return cleaned_data



from django.conf.urls import url

from exports.views import (
    AttendeeAccommExport, AttendeeBadgeExport, ChildCareExport, FoodExport,
    InvoiceExport, TalksExport, SpecialDietExport, BursaryExport,
    FingerprintExport,
)

urlpatterns = [
    url(r'^attendees/admin/export/accomm/$', AttendeeAccommExport.as_view(),
        name='attendee_admin_export_accomm'),
    url(r'^attendees/admin/export/badges/$', AttendeeBadgeExport.as_view(),
        name='attendee_admin_export_badges'),
    url(r'^attendees/admin/export/food/$', FoodExport.as_view(),
       name='attendee_admin_export_food'),
    url(r'^attendees/admin/export/special_diets/'
        r'(?P<date>[0-9-]+)/(?P<meal>[a-z]+)/$', SpecialDietExport.as_view()),
    url(r'^attendees/admin/export/child_care/$', ChildCareExport.as_view(),
        name='attendee_admin_export_childcare'),
    url(r'^attendees/admin/export/bursaries/$', BursaryExport.as_view(),
       name='attendee_admin_export_bursaries'),
    url(r'^talks/admin/export/$', TalksExport.as_view(),
        name='talks_admin_export'),
    url(r'^attendees/admin/export/fingerprints/$', FingerprintExport.as_view(),
        name='attendee_admin_export_fingerprints'),
    url(r'^attendees/admin/export/invoices/$', InvoiceExport.as_view(),
        name='attendee_admin_export_invoices'),
]

exports = [{'url': u.name, 'name': u.callback.view_class.name} for u in urlpatterns if u.name]



from __future__ import unicode_literals

from itertools import count

from django.db import migrations, models


temp_counter = count()
def temp_count():
    return next(temp_counter)


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0003_remove_accomm_special_needs'),
    ]

    operations = [
        migrations.AddField(
            model_name='queueslot',
            name='monotonic_position',
            field=models.IntegerField(default=temp_count),
            preserve_default=False,
        ),
    ]



import re

from django import forms
from django.conf import settings

from crispy_forms.helper import FormHelper


class ContactInformationForm(forms.Form):
    name = forms.CharField(
        label='My name is',
        help_text='This will appear on your name tag, and in public areas of '
                  'this site, e.g. if you submit a talk.',
        max_length=50,
    )
    # TODO: Consider storing the "wallet name" here, and having a separate
    # public name
    # TODO: nametag preview
    nametag_2 = forms.CharField(
        label='Nametag line 2',
        help_text="This could be your company, project, preferred pronoun or "
                  "anything you'd like to say.",
        max_length=50,
        required=False,
    )
    nametag_3 = forms.CharField(
        label='Nametag line 3',
        help_text="This could be your nick, username, or something "
                  "suitably silly.",
        max_length=50,
        required=False,
    )
    email = forms.EmailField(
        label='My e-mail address is',
        help_text="This won't be listed publicly.",
    )
    phone = forms.CharField(
        label='My contact number',
        help_text="The full number, including international dialing codes, "
                  "please. This won't be listed publicly.",
        max_length=16,
        required=False,
    )
    emergency_contact = forms.CharField(
        label='My emergency contact',
        help_text='Please include the name, full international phone number, '
                  'and language spoken (if not English).',
        widget=forms.Textarea(attrs={'rows': 3}),
        required=False,
    )
    # Purposefully left unchecked by default to make this opt-in.
    announce_me = forms.BooleanField(
        label='Announce my arrival',
        help_text='If checked, your name will be announced in the IRC channel '
                  'when you check in to the conference.',
        required=False,
    )
    register_announce = forms.BooleanField(
        label="Subscribe me to the DebConf-announce mailing list",
        help_text='This low-volume mailing list is the primary way for us to '
                  'reach attendees about important conference news and '
                  'information.',
        required=False,
        initial=True,
    )
    register_discuss = forms.BooleanField(
        label='Subscribe me to the DebConf-discuss mailing list',
        help_text='This mailing list is used by attendees and interested '
                  'people for general discussions about the conference.',
        required=False,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False
        if settings.DEBCONF_ONLINE:
            del self.fields['nametag_2']
            del self.fields['nametag_3']
            del self.fields['emergency_contact']
            del self.fields['announce_me']
        else:
            self.fields['emergency_contact'].required = True

    def clean_phone(self):
        phone = self.cleaned_data.get('phone')
        if phone and not re.match(r'^\+', phone):
            raise forms.ValidationError(
                "If you provide a phone number, please make sure it's in "
                "international dialling format. e.g. +1 234 567 8999")
        return phone



from django import forms
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import pgettext_lazy as _p
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Hidden, Submit, Button
from minidebconf.models import Registration


class RegisterForm(forms.ModelForm):
    full_name = forms.CharField(max_length=256, label=_('Full name'))
    class Meta:
        model = Registration
        fields = ['full_name', 'involvement', 'gender', 'country', 'city_state', 'days']
        widgets = {
            'days': forms.CheckboxSelectMultiple(),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["full_name"].initial = self.instance.full_name
        self.helper = FormHelper()

        if self.instance.id:
            submit = _p("conference", "Update registration")
        else:
            submit = _p("conference", "Register")
        self.helper.add_input(Submit("submit", submit))


    def save(self):
        super().save()
        name = self.cleaned_data['full_name'].split()
        user = self.instance.user
        user.first_name = name[0]
        user.last_name = " ".join(name[1:])
        user.save()



import datetime

from django import forms
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.utils import dateparse
from django.utils.encoding import force_text
from django.utils.timezone import make_naive, now

from crispy_forms.bootstrap import FormActions
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Button, Div, Fieldset, Layout, HTML, Submit

from wafer.schedule.models import ScheduleItem, Venue

import pytz

from register.models import Attendee
from volunteers.models import Task, TaskTemplate, Volunteer
from volunteers.utils import get_start_end_for_scheduleitem

# The granularity in minutes for the task filtering
MINUTE_GRANULARITY = 5


class DateTimeSelectWidget(forms.Widget):
    fields = ('day', 'hour', 'minute')

    day_field = '%s_day'
    hour_field = '%s_hour'
    minute_field = '%s_minute'

    input_type = 'select'
    select_widget = forms.widgets.Select

    template_name = 'wafer.volunteers/fields/selectdatetimefield.html'

    def __init__(self, attrs=None):
        self.attrs = attrs or {}

        self.days = []
        day = settings.VOLUNTEERS_FIRST_DAY
        while day < settings.VOLUNTEERS_LAST_DAY:
            self.days.append(day)
            day += datetime.timedelta(days=1)

        self.hours = [i for i in range(24)]

        self.minutes = []
        for x in range(60//MINUTE_GRANULARITY):
            minute = MINUTE_GRANULARITY * x
            self.minutes.append(minute)

    def get_context(self, name, value, attrs):
        context = super(DateTimeSelectWidget, self).get_context(
            name, value, attrs
        )
        datetime_context = {}

        day_attrs = context['widget']['attrs'].copy()
        day_name = self.day_field % name
        day_attrs['id'] = 'id_%s' % day_name
        day_choices = [(day, force_text(day)) for day in self.days]
        datetime_context['day'] = self.select_widget(
            attrs, choices=day_choices
        ).get_context(
            name=day_name,
            value=context['widget']['value']['day'],
            attrs=day_attrs,
        )

        hour_attrs = context['widget']['attrs'].copy()
        hour_name = self.hour_field % name
        hour_attrs['id'] = 'id_%s' % hour_name
        hour_choices = [(hour, force_text(hour)) for hour in self.hours]
        datetime_context['hour'] = self.select_widget(
            attrs, choices=hour_choices
        ).get_context(
            name=hour_name,
            value=context['widget']['value']['hour'],
            attrs=hour_attrs,
        )

        minute_attrs = context['widget']['attrs'].copy()
        minute_name = self.minute_field % name
        minute_attrs['id'] = 'id_%s' % minute_name
        minute_choices = [
            (minute, force_text(minute)) for minute in self.minutes
        ]
        datetime_context['minute'] = self.select_widget(
            attrs, choices=minute_choices
        ).get_context(
            name=minute_name,
            value=context['widget']['value']['minute'],
            attrs=minute_attrs,
        )
        subwidgets = []
        for field in self.fields:
            subwidgets.append(datetime_context[field]['widget'])
        context['widget']['subwidgets'] = subwidgets
        return context

    def value_from_datadict(self, data, files, name):
        day = data.get(self.day_field % name)
        hour = data.get(self.hour_field % name)
        minute = data.get(self.minute_field % name)

        if day and hour and minute:
            return str(
                datetime.datetime.combine(
                    dateparse.parse_date(day),
                    datetime.time(hour=int(hour), minute=int(minute)),
                )
            )

    def format_value(self, value):
        day, hour, minute = None, None, None
        if isinstance(value, (datetime.date, datetime.datetime)):
            day = value.date()
            if isinstance(value, datetime.datetime):
                hour = value.hour
                minute = value.minute
            else:
                hour = 0
                minute = 0
        elif isinstance(value, str):
            dt = dateparse.parse_datetime(value)
            day = dt.date()
            hour = dt.hour
            minute = dt.minute

        # Round down to next MINUTE_GRANULARITY
        if minute:
            minute = minute // MINUTE_GRANULARITY * MINUTE_GRANULARITY

        return {
            'day': day,
            'hour': hour,
            'minute': minute,
        }


class TasksFilterForm(forms.Form):
    start = forms.DateTimeField(label='Tasks starting', required=False,
                                widget=DateTimeSelectWidget())
    end = forms.DateTimeField(label='Tasks until', required=False,
                              widget=DateTimeSelectWidget())
    extra_filters = forms.MultipleChoiceField(
        label='Extra filters',
        choices=(
            ('needed', 'Volunteers needed'),
            ('preferred', 'Preferred tasks only'),
        ),
        required=False,
        widget=forms.CheckboxSelectMultiple(),
    )

    def __init__(self, *args, **kwargs):
        self.timezone = pytz.timezone(kwargs.pop('timezone'))
        start = now().astimezone(self.timezone)
        kwargs['initial'] = {
            'start': start,
            'end': start + datetime.timedelta(days=1),
            'extra_filters': 'needed',
        }

        super().__init__(*args, **kwargs)

        self.helper = FormHelper()
        self.helper.disable_csrf = True

        self.helper.layout = Layout(
            'start',
            'end',
            HTML('Timezone: ' + start.strftime('%z')),
            'extra_filters',
        )
        self.helper.add_input(
            Submit('submit', 'Update filter', css_class='btn btn-success')
        )

    def clean_start(self):
        # DateTimeField applied system tzinfo, not the volunteer timezone
        return self.timezone.localize(make_naive(self.cleaned_data['start']))

    def clean_end(self):
        return self.timezone.localize(make_naive(self.cleaned_data['end']))


class VideoMassCreateTaskForm(forms.Form):
    def __init__(self, *args, **kwargs):
        lead_time = kwargs.pop('lead_time')
        super().__init__(*args, **kwargs)

        video_tasks = TaskTemplate.objects.filter(video_task=True)
        schedule_items = list(
            ScheduleItem.objects.filter(
                venue__video=True,
                talk__video=True,
            ).all()
        )
        venues = Venue.objects.filter(video=True).all()
        schedule_items.sort(key=lambda si: si.get_start_datetime())

        venue_divs = {venue.pk: Div(HTML('<h3>%s</h3>' % venue.name),
                                    css_class='venue-%s' % venue.pk)
                      for venue in venues}

        starting_after = now() + lead_time
        for schedule_item in schedule_items:
            if schedule_item.get_start_datetime() < starting_after:
                continue
            tasks_for_item = Fieldset(legend='%s - %s - %s' % (
                schedule_item.venue.name,
                schedule_item.get_start_time(),
                schedule_item.talk.title,
            ))
            for task in video_tasks:
                fieldname = '%s:%s' % (schedule_item.pk, task.pk)
                initial = Task.objects.filter(schedule_item=schedule_item,
                                              template=task).exists()
                self.fields[fieldname] = forms.BooleanField(
                    required=False,
                    label=task.name,
                    initial=initial,
                )
                tasks_for_item.append(fieldname)
            venue_divs[schedule_item.venue.pk].append(tasks_for_item)

        self.helper = FormHelper()
        self.helper.layout = Layout(*venue_divs.values())
        self.helper.add_input(Button('select_all', 'Select All'))
        self.helper.add_input(Submit('submit', 'Create Tasks'))

    def clean(self):
        cleaned_data = super().clean()
        new_data = {}
        for pks, create_task in cleaned_data.items():
            schedule_item_pk, task_template_pk = map(int, pks.split(':'))
            si = ScheduleItem.objects.get(pk=schedule_item_pk)
            template = TaskTemplate.objects.get(pk=task_template_pk)

            start, end = get_start_end_for_scheduleitem(si)

            task_data = {
                'start': start,
                'end': end,
                'venue': si.venue,
                'nbr_volunteers_min': None,
                'nbr_volunteers_max': None,
            }

            if create_task:
                task = Task.objects.get_or_create(schedule_item=si,
                                                  template=template,
                                                  defaults=task_data)
                new_data[(si, template)] = task
            else:
                Task.objects.filter(
                    schedule_item=si, template=template).delete()
                new_data[(si, template)] = None
        cleaned_data.update(new_data)
        return cleaned_data


class VideoShirtForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper()
        fields = []
        volunteers = sorted(self.eligable_volunteers(),
                            key=lambda v: v['tasks'], reverse=True)
        for volunteer in volunteers:
            fieldname = 'volunteer:{username}'.format(**volunteer)
            self.fields[fieldname] = forms.BooleanField(
                required=False,
                label='{tasks}: {username}: {name} ({shirt_size})'.format(
                    **volunteer),
                initial=False,
            )
            fields.append(fieldname)

        self.helper.layout = Layout(*fields)
        self.helper.add_input(Submit('submit', 'Mark as Received'))

    def eligable_volunteers(self):
        shirt_sizes = dict(settings.DEBCONF_T_SHIRT_SIZES)
        video_admin_group = Group.objects.get_by_natural_key('Volunteer Admin')
        for volunteer in Volunteer.objects.select_related(
                    'user',
                    'user__attendee',
                    'user__userprofile',
                ).all():
            user = volunteer.user
            try:
                attendee = user.attendee
            except Attendee.DoesNotExist:
                continue

            # Already issued?
            video_shirt_kv = user.userprofile.kv.filter(
                group=video_admin_group, key='video_shirt').first()
            video_shirt = video_shirt_kv and video_shirt_kv.value

            if video_shirt:
                continue

            v = {
                'username': user.username,
                'name': user.userprofile.display_name(),
                'shirt_size': shirt_sizes[attendee.t_shirt_size],
                'tasks': 0
            }
            for task in volunteer.tasks.all():
                template = task.template
                if not template or not template.video_task:
                    continue
                if task.start > now():
                    continue
                v['tasks'] += 1

            if v['tasks'] >= 3:
                yield v

    def clean(self):
        cleaned_data = super().clean()
        new_data = {}
        video_admin_group = Group.objects.get_by_natural_key('Volunteer Admin')
        for volunteer, issued in cleaned_data.items():
            if not issued:
                continue
            username = volunteer.split(':', 1)[1]
            user = get_user_model().objects.get(username=username)
            user.userprofile.kv.get_or_create(
                group=video_admin_group, key='video_shirt',
                defaults={'value': True})
        cleaned_data.update(new_data)
        return cleaned_data


class VolunteerUpdateForm(forms.ModelForm):
    class Meta:
        model = Volunteer
        fields = ['preferred_categories', 'preferred_task_types', 'timezone']
        widgets = {
            'preferred_categories': forms.CheckboxSelectMultiple(),
            'preferred_task_types': forms.CheckboxSelectMultiple(),
            'timezone': forms.Select(),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper()
        self.helper.layout = Layout(
            'preferred_categories',
            'preferred_task_types',
            'timezone',
            FormActions(
                Submit('submit', 'Update preferences',
                       css_class='btn btn-success')
            ),
        )



#!/usr/bin/env python
from glob import glob
from pathlib import Path
from setuptools import find_packages, setup
from setuptools.command.build_py import build_py
import subprocess

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()


class CustomBuild(build_py):
    assets = (
        (
            "libjs-bootstrap4",
            "4.3",
            [
                "debconf/static/vendor/bootstrap/scss",
                "debconf/static/vendor/bootstrap/js",
            ],
        ),
        (
            "fonts-fork-awesome",
            "1.1",
            [
                "debconf/static/vendor/fork-awesome/scss",
                "debconf/static/vendor/fork-awesome/fonts",
            ],
        ),
        ("libjs-jquery", "3", ["debconf/static/vendor/jquery/jquery.js"]),
        ("libjs-moment", "2.28", ["debconf/static/vendor/moment/moment-with-locales.js"]),
        ("libjs-moment-timezone", "0.5.31+dfsg1-2", ["debconf/static/vendor/moment-timezone/moment-timezone-with-data.js"]),
    )

    def check_assets(self):
        base = Path(__file__).parent
        missing_packages = []
        for package, version, links in self.assets:
            broken_links = [l for l in links if not (base / l).exists()]
            if broken_links:
                try:
                    self.check_deb_package(package, version)
                except RuntimeError as exc:
                    missing_packages.append(str(exc))
        if missing_packages:
            raise RuntimeError("; ".join(missing_packages))

    def check_deb_package(self, package, version):
        try:
            _, instversion = subprocess.check_output(["dpkg-query", "--show", package]).split()
        except subprocess.CalledProcessError:
            raise RuntimeError(f"{package} not installed")
        try:
            subprocess.check_call(["dpkg", "--compare-versions", instversion, "ge", version])
        except subprocess.CalledProcessError:
            raise RuntimeError(f"{package} (>= {version}) required, found {instversion}")

    def run(self):
        self.check_assets()
        super().run()


def compile_translations():
    try:
        subprocess.check_call(['./manage.py', 'compilemessages'])
    except subprocess.CalledProcessError:
        print("WARNING: cannot compile translations.")
        pass
    return glob('*/locale/*/LC_MESSAGES/django.mo')


setup(
    name='wafer-debconf',
    version='0.6.1',
    description='Wafer extensions used by DebConf',
    author='DebConf Team',
    author_email='debconf-team@lists.debian.org',
    url='https://salsa.debian.org/debconf-team/public/websites/wafer-debconf',
    packages=find_packages(),
    cmdclass={
        "build_py": CustomBuild,
    },
    include_package_data=True,
    package_data={
        "debconf": ["static/vendor/*", "static/vendor/*/*", "static/vendor/*/*/*"],
    },
    data_files=compile_translations(),
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        "mdx_linkify",
        "wafer>=0.11",
        "mdx_staticfiles",
        "Django>=2,<3",
        "django_compressor",
        "django_countries",
        "django_extensions",
        "django_libsass",
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Communications :: Conferencing',
    ],
)



from django.core.management.base import BaseCommand
from django.db.models import F, Q
from functools import reduce
from random import shuffle
from datetime import timedelta


from wafer.talks.models import Talk, ACCEPTED
from wafer.schedule.models import Slot, Venue, ScheduleItem


def commaseparated(s):
    return s.split(",")


class Command(BaseCommand):
    help = "Schedule selected talks"

    def add_arguments(self, parser):
        parser.add_argument(
            "--language",
            type=commaseparated,
            help="Select only talks from LANGUAGE (single language, or comma-separated)",
        )
        parser.add_argument(
            "--talk-type",
            action="append",
            help="Select only talks from the named talk type",
        )
        parser.add_argument("--limit", type=int, help="Schedule only LIMIT items")
        parser.add_argument("--start", help="when to start scheduling")
        parser.add_argument("--end", help="when to end scheduling")
        parser.add_argument("--duration", type=int, help="select only slots with duration of DURATION minutes")
        parser.add_argument("--venue", action="append", help="Schedule only to VENUE")
        parser.add_argument(
            "--yes",
            action="store_true",
            help="Really schedule (otherwise just print what would have been done)",
        )

    def handle(self, *args, **options):
        talks = Talk.objects.filter(
            status=ACCEPTED, scheduleitem__isnull=True
        ).order_by("talk_id")
        if options["language"]:
            talks = talks.filter(language__in=options["language"])
        if options["talk_type"]:
            name_filters = [Q(talk_type__name__icontains=v) for v in options["talk_type"]]
            condition = reduce(lambda x, y: x | y, name_filters)
            talks = talks.filter(condition)

        valid_slots = []

        slots = Slot.objects.all()
        if options["start"]:
            slots = slots.filter(start_time__gte=options["start"])
        if options["end"]:
            slots = slots.filter(end_time__lte=options["end"])
        if options["duration"]:
            duration = timedelta(minutes=options["duration"])
            slots = [
                s for s in slots
                if s.end_time - s.start_time == duration
            ]

        venues = Venue.objects.all().filter(video=True).order_by("id")
        if options["venue"]:
            venues = venues.filter(name__icontains=options["venue"])

        for slot in slots:
            if ScheduleItem.objects.filter(slots=slot, expand=True).exists():
                continue
            for venue in venues:
                if ScheduleItem.objects.filter(slots=slot, venue=venue).exists():
                    continue
                valid_slots.append((slot, venue))

        shuffle(valid_slots)
        available_slots = iter(valid_slots)

        count = 0
        for talk in talks:
            try:
                slot, venue = next(available_slots)
            except StopIteration:
                print(f"W: no slot left to schedule \"{talk}\"")
                continue
            print(f"Scheduling \"{talk}\" to {slot} in {venue}")
            if options["yes"]:
                item = ScheduleItem.objects.create(
                    venue=venue,
                    talk=talk,
                )
                item.slots.add(slot)
            count += 1
            if options["limit"] and count >= options["limit"]:
                break



import sys


from django.contrib.sites.models import Site
from django.conf import settings
from django.core.management.base import BaseCommand


from register.models import Attendee
from wafer.talks.models import Talk
from wafer.talks.models import ACCEPTED


class Command(BaseCommand):
    help = "List country speakers for each talk."

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        talks = Talk.objects.filter(
            status=ACCEPTED,
            scheduleitem=None,
        ).order_by(
            "title"
        ).prefetch_related(
            "authors",
        )
        site = Site.objects.first().domain
        for talk in talks:
            print(talk.title)
            print('-' * len(talk.title))
            print(f"Link: http://{site}{talk.get_absolute_url()}")
            for author in talk.authors.all():
                name = author.get_full_name() or author.username
                try:
                    country = author.attendee.country_name
                except Attendee.DoesNotExist:
                    country = "?"
                print(f"{name}: {country}")
            if talk.notes:
                print(f"Notes: {talk.notes}")
            print("")
            print("")




# Generated by Django 2.2.14 on 2020-07-05 23:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bursary', '0002_update_fk'),
    ]

    operations = [
        migrations.AddField(
            model_name='bursary',
            name='expenses_status',
            field=models.CharField(choices=[('submitted', 'The bursary request has been submitted'), ('ranked', 'The bursary request has been ranked'), ('pending', 'The bursary request has been granted and is pending your acceptance'), ('accepted', 'The bursary has been accepted'), ('denied', 'The bursary has been denied'), ('expired', 'The bursary grant has expired'), ('canceled', 'The bursary request has been canceled')], default='submitted', max_length=32),
        ),
        migrations.AddField(
            model_name='bursary',
            name='request_expenses',
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
    ]
